import React, { Component, createRef } from 'react';
import PropTypes$1, { PropTypes } from 'prop-types';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};
  var target = _objectWithoutPropertiesLoose(source, excluded);
  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

const LIKE = 'LIKE';
const EQ = '=';
const NE = '!=';
const GT = '>';
const GE = '>=';
const LT = '<';
const LE = '<=';

var Comparison = /*#__PURE__*/Object.freeze({
  __proto__: null,
  LIKE: LIKE,
  EQ: EQ,
  NE: NE,
  GT: GT,
  GE: GE,
  LT: LT,
  LE: LE
});

const FILTER_TYPE = {
  TEXT: 'TEXT',
  SELECT: 'SELECT',
  MULTISELECT: 'MULTISELECT',
  NUMBER: 'NUMBER',
  DATE: 'DATE'
};
const FILTER_DELAY = 500;

function getDefaultValue(props) {
  if (props.filterState && typeof props.filterState.filterVal !== 'undefined') {
    return props.filterState.filterVal;
  }

  return props.defaultValue;
}

class TextFilter extends Component {
  constructor(props) {
    super(props);
    this.inputRef = /*#__PURE__*/createRef();

    this.handleClick = e => {
      const {
        onClick
      } = this.props;
      e.stopPropagation();

      if (onClick) {
        onClick(e);
      }
    };

    this.filter = e => {
      const {
        onFilter,
        column,
        delay
      } = this.props;
      e.stopPropagation();
      this.cleanTimer();
      const filterValue = e.target.value;
      this.setState(() => ({
        value: filterValue
      }));
      this.timeout = setTimeout(() => {
        onFilter(column, FILTER_TYPE.TEXT)(filterValue);
      }, delay);
    };

    this.timeout = null;
    this.state = {
      value: getDefaultValue(props)
    };
  }

  componentDidMount() {
    const {
      onFilter,
      getFilter,
      column
    } = this.props;
    const defaultValue = this.inputRef.current.value;

    if (defaultValue) {
      onFilter(column, FILTER_TYPE.TEXT, true)(defaultValue);
    } // export onFilter function to allow users to access


    if (getFilter) {
      getFilter(filterVal => {
        this.setState(() => ({
          value: filterVal
        }));
        onFilter(column, FILTER_TYPE.TEXT)(filterVal);
      });
    }
  } // UNSAFE_componentWillReceiveProps(nextProps) {


  componentDidUpdate(prevProps) {
    if (this.props.defaultValue !== prevProps.defaultValue) {
      this.applyFilter(this.props.defaultValue);
    }
  }

  componentWillUnmount() {
    this.cleanTimer();
  }

  cleanTimer() {
    if (this.timeout) {
      clearTimeout(this.timeout);
    }
  }

  applyFilter(filterText) {
    const {
      onFilter,
      column
    } = this.props;
    this.setState(() => ({
      value: filterText
    }));
    onFilter(column, FILTER_TYPE.TEXT)(filterText);
  }

  cleanFiltered() {
    const {
      defaultValue: value,
      onFilter,
      column
    } = this.props;
    this.setState(() => ({
      value
    }));
    onFilter(column, FILTER_TYPE.TEXT)(value);
  }

  render() {
    const _this$props = this.props,
          {
      id,
      placeholder,
      column: {
        dataField,
        text
      },
      style,
      className,
      onFilter,
      caseSensitive,
      defaultValue,
      getFilter,
      filterState
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["id", "placeholder", "column", "style", "className", "onFilter", "caseSensitive", "defaultValue", "getFilter", "filterState"]);

    const {
      value
    } = this.state;
    const elmId = `text-filter-column-${dataField}${id ? `-${id}` : ''}`;
    return /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: elmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Filter by", text), /*#__PURE__*/React.createElement("input", _extends({}, rest, {
      // ref={ (n) => this.input = n }
      ref: this.inputRef,
      type: "text",
      id: elmId,
      className: `filter text-filter form-control ${className}`,
      style: style,
      onChange: this.filter,
      onClick: this.handleClick,
      placeholder: placeholder || `Enter ${text}...`,
      value: value
    })));
  }

}

TextFilter.propTypes = {
  onFilter: PropTypes.func.isRequired,
  column: PropTypes.object.isRequired,
  id: PropTypes.string,
  filterState: PropTypes.object,
  comparator: PropTypes.oneOf([LIKE, EQ]),
  defaultValue: PropTypes.string,
  delay: PropTypes.number,
  placeholder: PropTypes.string,
  style: PropTypes.object,
  className: PropTypes.string,
  caseSensitive: PropTypes.bool,
  getFilter: PropTypes.func
};
TextFilter.defaultProps = {
  delay: FILTER_DELAY,
  filterState: {},
  defaultValue: '',
  caseSensitive: false,
  id: null
};

function optionsEquals(currOpts, prevOpts) {
  if (Array.isArray(currOpts)) {
    if (currOpts.length === prevOpts.length) {
      for (let i = 0; i < currOpts.length; i += 1) {
        if (currOpts[i].value !== prevOpts[i].value || currOpts[i].label !== prevOpts[i].label) {
          return false;
        }
      }

      return true;
    }

    return false;
  }

  const keys = Object.keys(currOpts);

  for (let i = 0; i < keys.length; i += 1) {
    if (currOpts[keys[i]] !== prevOpts[keys[i]]) {
      return false;
    }
  }

  return Object.keys(currOpts).length === Object.keys(prevOpts).length;
}

function getOptionValue(options, key) {
  if (Array.isArray(options)) {
    const result = options.filter(({
      label
    }) => label === key).map(({
      value
    }) => value);
    return result[0];
  }

  return options[key];
}

class SelectFilter extends Component {
  constructor(props) {
    super(props);

    this.filter = e => {
      const {
        onFilter,
        column
      } = this.props;
      const {
        value
      } = e.target;
      this.setState(() => ({
        isSelected: value !== ''
      }));
      onFilter(column, FILTER_TYPE.SELECT)(value);
    };

    this.options = this.getOptions(props);
    const isSelected = getOptionValue(this.options, this.getDefaultValue()) !== undefined;
    this.state = {
      isSelected
    };
  }

  componentDidMount() {
    const {
      column,
      onFilter,
      getFilter
    } = this.props;
    const {
      value
    } = this.selectInput;

    if (value && value !== '') {
      onFilter(column, FILTER_TYPE.SELECT, true)(value);
    } // export onFilter function to allow users to access


    if (getFilter) {
      getFilter(filterVal => {
        this.setState(() => ({
          isSelected: filterVal !== ''
        }));
        this.selectInput.value = filterVal;
        onFilter(column, FILTER_TYPE.SELECT)(filterVal);
      });
    }
  }

  componentDidUpdate(prevProps) {
    let needFilter = false;
    const {
      column,
      onFilter,
      defaultValue
    } = this.props;
    const nextOptions = this.getOptions(this.props);

    if (defaultValue !== prevProps.defaultValue) {
      needFilter = true;
    } else if (!optionsEquals(nextOptions, this.options)) {
      this.options = nextOptions;
      needFilter = true;
    }

    if (needFilter) {
      const {
        value
      } = this.selectInput;

      if (value) {
        onFilter(column, FILTER_TYPE.SELECT)(value);
      }
    }
  }

  getOptions(props) {
    return typeof props.options === 'function' ? props.options(props.column) : props.options;
  }

  getDefaultValue() {
    const {
      filterState,
      defaultValue
    } = this.props;

    if (filterState && typeof filterState.filterVal !== 'undefined') {
      return filterState.filterVal;
    }

    return defaultValue;
  }

  cleanFiltered() {
    const {
      defaultValue,
      onFilter,
      column
    } = this.props;
    const value = defaultValue !== undefined ? defaultValue : '';
    this.setState(() => ({
      isSelected: value !== ''
    }));
    this.selectInput.value = value;
    onFilter(column, FILTER_TYPE.SELECT)(value);
  }

  applyFilter(value) {
    const {
      onFilter,
      column
    } = this.props;
    this.selectInput.value = value;
    this.setState(() => ({
      isSelected: value !== ''
    }));
    onFilter(column, FILTER_TYPE.SELECT)(value);
  }

  renderOptions() {
    const optionTags = [];
    const {
      options
    } = this;
    const {
      placeholder,
      column,
      withoutEmptyOption
    } = this.props;

    if (!withoutEmptyOption) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: "-1",
        value: ""
      }, placeholder || `Select ${column.text}...`));
    }

    if (Array.isArray(options)) {
      // eslint-disable-next-line max-len
      options.forEach(({
        value,
        label
      }) => optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: value,
        value: value
      }, label)));
    } else {
      // eslint-disable-next-line max-len
      Object.keys(options).forEach(key => optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: key,
        value: key
      }, options[key])));
    }

    return optionTags;
  }

  render() {
    const _this$props = this.props,
          {
      id,
      style,
      className,
      defaultValue,
      onFilter,
      column,
      options,
      comparator,
      withoutEmptyOption,
      caseSensitive,
      getFilter,
      filterState
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["id", "style", "className", "defaultValue", "onFilter", "column", "options", "comparator", "withoutEmptyOption", "caseSensitive", "getFilter", "filterState"]);

    const {
      isSelected
    } = this.state;
    const selectClass = `filter select-filter form-control ${className} ${isSelected ? '' : 'placeholder-selected'}`;
    const elmId = `select-filter-column-${column.dataField}${id ? `-${id}` : ''}`;
    return /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: elmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Filter by", column.text), /*#__PURE__*/React.createElement("select", _extends({}, rest, {
      ref: n => this.selectInput = n,
      id: elmId,
      style: style,
      className: selectClass,
      onChange: this.filter,
      onClick: e => e.stopPropagation(),
      defaultValue: this.getDefaultValue() || ''
    }), this.renderOptions()));
  }

}

SelectFilter.propTypes = {
  onFilter: PropTypes$1.func.isRequired,
  column: PropTypes$1.object.isRequired,
  id: PropTypes$1.string,
  filterState: PropTypes$1.object,
  options: PropTypes$1.oneOfType([PropTypes$1.object, PropTypes$1.array]).isRequired,
  comparator: PropTypes$1.oneOf([LIKE, EQ]),
  placeholder: PropTypes$1.string,
  style: PropTypes$1.object,
  className: PropTypes$1.string,
  withoutEmptyOption: PropTypes$1.bool,
  defaultValue: PropTypes$1.any,
  caseSensitive: PropTypes$1.bool,
  getFilter: PropTypes$1.func
};
SelectFilter.defaultProps = {
  defaultValue: '',
  filterState: {},
  className: '',
  withoutEmptyOption: false,
  comparator: EQ,
  caseSensitive: true,
  id: null
};

function optionsEquals$1(currOpts, prevOpts) {
  const keys = Object.keys(currOpts);

  for (let i = 0; i < keys.length; i += 1) {
    if (currOpts[keys[i]] !== prevOpts[keys[i]]) {
      return false;
    }
  }

  return Object.keys(currOpts).length === Object.keys(prevOpts).length;
}

const getSelections = container => {
  if (container.selectedOptions) {
    return Array.from(container.selectedOptions).map(item => item.value);
  }

  const selections = [];
  const totalLen = container.options.length;

  for (let i = 0; i < totalLen; i += 1) {
    const option = container.options.item(i);
    if (option.selected) selections.push(option.value);
  }

  return selections;
};

class MultiSelectFilter extends Component {
  constructor(props) {
    super(props);

    this.applyFilter = value => {
      const {
        onFilter,
        column
      } = this.props;

      if (value.length === 1 && value[0] === '') {
        value = [];
      }

      this.setState(() => ({
        isSelected: value.length > 0
      }));
      onFilter(column, FILTER_TYPE.MULTISELECT)(value);
    };

    this.filter = e => {
      const value = getSelections(e.target);
      this.applyFilter(value);
    };

    this.applyFilter = this.applyFilter.bind(this);
    const isSelected = props.defaultValue.map(item => props.options[item]).length > 0;
    this.state = {
      isSelected
    };
  }

  componentDidMount() {
    const {
      getFilter
    } = this.props;
    const value = getSelections(this.selectInput);

    if (value && value.length > 0) {
      this.applyFilter(value);
    } // export onFilter function to allow users to access


    if (getFilter) {
      getFilter(filterVal => {
        this.selectInput.value = filterVal;
        this.applyFilter(filterVal);
      });
    }
  }

  componentDidUpdate(prevProps) {
    const {
      defaultValue,
      options
    } = this.props;
    let needFilter = false;

    if (defaultValue !== prevProps.defaultValue) {
      needFilter = true;
    } else if (!optionsEquals$1(options, prevProps.options)) {
      needFilter = true;
    }

    if (needFilter) {
      this.applyFilter(getSelections(this.selectInput));
    }
  }

  getDefaultValue() {
    const {
      filterState,
      defaultValue
    } = this.props;

    if (filterState && typeof filterState.filterVal !== 'undefined') {
      return filterState.filterVal;
    }

    return defaultValue;
  }

  getOptions() {
    const optionTags = [];
    const {
      options,
      placeholder,
      column,
      withoutEmptyOption
    } = this.props;

    if (!withoutEmptyOption) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: "-1",
        value: ""
      }, placeholder || `Select ${column.text}...`));
    } // eslint-disable-next-line max-len


    Object.keys(options).forEach(key => optionTags.push( /*#__PURE__*/React.createElement("option", {
      key: key,
      value: key
    }, options[key])));
    return optionTags;
  }

  cleanFiltered() {
    const {
      defaultValue
    } = this.props;
    const value = defaultValue !== undefined ? defaultValue : [];
    this.selectInput.value = value;
    this.applyFilter(value);
  }

  render() {
    const _this$props = this.props,
          {
      id,
      style,
      className,
      filterState,
      defaultValue,
      onFilter,
      column,
      options,
      comparator,
      withoutEmptyOption,
      caseSensitive,
      getFilter
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, ["id", "style", "className", "filterState", "defaultValue", "onFilter", "column", "options", "comparator", "withoutEmptyOption", "caseSensitive", "getFilter"]);

    const {
      isSelected
    } = this.state;
    const selectClass = `filter select-filter form-control ${className} ${isSelected ? '' : 'placeholder-selected'}`;
    const elmId = `multiselect-filter-column-${column.dataField}${id ? `-${id}` : ''}`;
    return /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: elmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Filter by", column.text), /*#__PURE__*/React.createElement("select", _extends({}, rest, {
      ref: n => this.selectInput = n,
      id: elmId,
      style: style,
      multiple: true,
      className: selectClass,
      onChange: this.filter,
      onClick: e => e.stopPropagation(),
      defaultValue: this.getDefaultValue()
    }), this.getOptions()));
  }

}

MultiSelectFilter.propTypes = {
  onFilter: PropTypes$1.func.isRequired,
  column: PropTypes$1.object.isRequired,
  options: PropTypes$1.object.isRequired,
  id: PropTypes$1.string,
  filterState: PropTypes$1.object,
  comparator: PropTypes$1.oneOf([LIKE, EQ]),
  placeholder: PropTypes$1.string,
  style: PropTypes$1.object,
  className: PropTypes$1.string,
  withoutEmptyOption: PropTypes$1.bool,
  defaultValue: PropTypes$1.array,
  caseSensitive: PropTypes$1.bool,
  getFilter: PropTypes$1.func
};
MultiSelectFilter.defaultProps = {
  defaultValue: [],
  filterState: {},
  className: '',
  withoutEmptyOption: false,
  comparator: EQ,
  caseSensitive: true,
  id: null
};

/* eslint-disable jsx-a11y/click-events-have-key-events */
const legalComparators = [EQ, NE, GT, GE, LT, LE];

class NumberFilter extends Component {
  constructor(props) {
    super(props);
    this.comparators = props.comparators || legalComparators;
    this.timeout = null;
    let isSelected = props.defaultValue !== undefined && props.defaultValue.number !== undefined;

    if (props.options && isSelected) {
      isSelected = props.options.indexOf(props.defaultValue.number) > -1;
    }

    this.state = {
      isSelected
    };
    this.onChangeNumber = this.onChangeNumber.bind(this);
    this.onChangeNumberSet = this.onChangeNumberSet.bind(this);
    this.onChangeComparator = this.onChangeComparator.bind(this);
  }

  componentDidMount() {
    const {
      column,
      onFilter,
      getFilter
    } = this.props;
    const comparator = this.numberFilterComparator.value;
    const number = this.numberFilter.value;

    if (comparator && number) {
      onFilter(column, FILTER_TYPE.NUMBER, true)({
        number,
        comparator
      });
    } // export onFilter function to allow users to access


    if (getFilter) {
      getFilter(filterVal => {
        this.setState(() => ({
          isSelected: filterVal !== ''
        }));
        this.numberFilterComparator.value = filterVal.comparator;
        this.numberFilter.value = filterVal.number;
        onFilter(column, FILTER_TYPE.NUMBER)({
          number: filterVal.number,
          comparator: filterVal.comparator
        });
      });
    }
  }

  componentWillUnmount() {
    clearTimeout(this.timeout);
  }

  onChangeNumber(e) {
    const {
      delay,
      column,
      onFilter
    } = this.props;
    const comparator = this.numberFilterComparator.value;

    if (comparator === '') {
      return;
    }

    if (this.timeout) {
      clearTimeout(this.timeout);
    }

    const filterValue = e.target.value;
    this.timeout = setTimeout(() => {
      onFilter(column, FILTER_TYPE.NUMBER)({
        number: filterValue,
        comparator
      });
    }, delay);
  }

  onChangeNumberSet(e) {
    const {
      column,
      onFilter
    } = this.props;
    const comparator = this.numberFilterComparator.value;
    const {
      value
    } = e.target;
    this.setState(() => ({
      isSelected: value !== ''
    })); // if (comparator === '') {
    //   return;
    // }

    onFilter(column, FILTER_TYPE.NUMBER)({
      number: value,
      comparator
    });
  }

  onChangeComparator(e) {
    const {
      column,
      onFilter
    } = this.props;
    const {
      value
    } = this.numberFilter;
    const comparator = e.target.value; // if (value === '') {
    //   return;
    // }

    onFilter(column, FILTER_TYPE.NUMBER)({
      number: value,
      comparator
    });
  }

  getDefaultComparator() {
    const {
      defaultValue,
      filterState
    } = this.props;

    if (filterState && filterState.filterVal) {
      return filterState.filterVal.comparator;
    }

    if (defaultValue && defaultValue.comparator) {
      return defaultValue.comparator;
    }

    return '';
  }

  getDefaultValue() {
    const {
      defaultValue,
      filterState
    } = this.props;

    if (filterState && filterState.filterVal) {
      return filterState.filterVal.number;
    }

    if (defaultValue && defaultValue.number) {
      return defaultValue.number;
    }

    return '';
  }

  getComparatorOptions() {
    const optionTags = [];
    const {
      withoutEmptyComparatorOption
    } = this.props;

    if (!withoutEmptyComparatorOption) {
      // eslint-disable-next-line jsx-a11y/control-has-associated-label
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: "-1"
      }));
    }

    for (let i = 0; i < this.comparators.length; i += 1) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: i,
        value: this.comparators[i]
      }, this.comparators[i]));
    }

    return optionTags;
  }

  getNumberOptions() {
    const optionTags = [];
    const {
      options,
      column,
      withoutEmptyNumberOption,
      placeholder
    } = this.props;

    if (!withoutEmptyNumberOption) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: "-1",
        value: ""
      }, placeholder || `Select ${column.text}...`));
    }

    for (let i = 0; i < options.length; i += 1) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: i,
        value: options[i]
      }, options[i]));
    }

    return optionTags;
  }

  applyFilter(filterObj) {
    const {
      column,
      onFilter
    } = this.props;
    const {
      number,
      comparator
    } = filterObj;
    this.setState(() => ({
      isSelected: number !== ''
    }));
    this.numberFilterComparator.value = comparator;
    this.numberFilter.value = number;
    onFilter(column, FILTER_TYPE.NUMBER)({
      number,
      comparator
    });
  }

  cleanFiltered() {
    const {
      column,
      onFilter,
      defaultValue
    } = this.props;
    const value = defaultValue ? defaultValue.number : '';
    const comparator = defaultValue ? defaultValue.comparator : '';
    this.setState(() => ({
      isSelected: value !== ''
    }));
    this.numberFilterComparator.value = comparator;
    this.numberFilter.value = value;
    onFilter(column, FILTER_TYPE.NUMBER)({
      number: value,
      comparator
    });
  }

  render() {
    const {
      isSelected
    } = this.state;
    const {
      id,
      column,
      options,
      style,
      className,
      numberStyle,
      numberClassName,
      comparatorStyle,
      comparatorClassName,
      placeholder
    } = this.props;
    const selectClass = `
      select-filter 
      number-filter-input 
      form-control 
      ${numberClassName} 
      ${!isSelected ? 'placeholder-selected' : ''}
    `;
    const comparatorElmId = `number-filter-comparator-${column.dataField}${id ? `-${id}` : ''}`;
    const inputElmId = `number-filter-column-${column.dataField}${id ? `-${id}` : ''}`;
    return /*#__PURE__*/React.createElement("div", {
      onClick: e => e.stopPropagation(),
      className: `filter number-filter ${className}`,
      style: style
    }, /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: comparatorElmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Filter comparator"), /*#__PURE__*/React.createElement("select", {
      ref: n => this.numberFilterComparator = n,
      style: comparatorStyle,
      id: comparatorElmId,
      className: `number-filter-comparator form-control ${comparatorClassName}`,
      onChange: this.onChangeComparator,
      defaultValue: this.getDefaultComparator()
    }, this.getComparatorOptions())), options ? /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: inputElmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, `Select ${column.text}`), /*#__PURE__*/React.createElement("select", {
      ref: n => this.numberFilter = n,
      id: inputElmId,
      style: numberStyle,
      className: selectClass,
      onChange: this.onChangeNumberSet,
      defaultValue: this.getDefaultValue()
    }, this.getNumberOptions())) : /*#__PURE__*/React.createElement("label", {
      htmlFor: inputElmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, `Enter ${column.text}`), /*#__PURE__*/React.createElement("input", {
      ref: n => this.numberFilter = n,
      id: inputElmId,
      type: "number",
      style: numberStyle,
      className: `number-filter-input form-control ${numberClassName}`,
      placeholder: placeholder || `Enter ${column.text}...`,
      onChange: this.onChangeNumber,
      defaultValue: this.getDefaultValue()
    })));
  }

}

NumberFilter.propTypes = {
  onFilter: PropTypes$1.func.isRequired,
  column: PropTypes$1.object.isRequired,
  id: PropTypes$1.string,
  filterState: PropTypes$1.object,
  options: PropTypes$1.arrayOf(PropTypes$1.number),
  defaultValue: PropTypes$1.shape({
    number: PropTypes$1.oneOfType([PropTypes$1.string, PropTypes$1.number]),
    comparator: PropTypes$1.oneOf([...legalComparators, ''])
  }),
  delay: PropTypes$1.number,

  /* eslint consistent-return: 0 */
  comparators: (props, propName) => {
    if (!props[propName]) {
      return;
    }

    for (let i = 0; i < props[propName].length; i += 1) {
      let comparatorIsValid = false;

      for (let j = 0; j < legalComparators.length; j += 1) {
        if (legalComparators[j] === props[propName][i] || props[propName][i] === '') {
          comparatorIsValid = true;
          break;
        }
      }

      if (!comparatorIsValid) {
        return new Error(`Number comparator provided is not supported.
          Use only ${legalComparators}`);
      }
    }
  },
  placeholder: PropTypes$1.string,
  withoutEmptyComparatorOption: PropTypes$1.bool,
  withoutEmptyNumberOption: PropTypes$1.bool,
  style: PropTypes$1.object,
  className: PropTypes$1.string,
  comparatorStyle: PropTypes$1.object,
  comparatorClassName: PropTypes$1.string,
  numberStyle: PropTypes$1.object,
  numberClassName: PropTypes$1.string,
  getFilter: PropTypes$1.func
};
NumberFilter.defaultProps = {
  delay: FILTER_DELAY,
  options: undefined,
  defaultValue: {
    number: undefined,
    comparator: ''
  },
  filterState: {},
  withoutEmptyComparatorOption: false,
  withoutEmptyNumberOption: false,
  comparators: legalComparators,
  placeholder: undefined,
  style: undefined,
  className: '',
  comparatorStyle: undefined,
  comparatorClassName: '',
  numberStyle: undefined,
  numberClassName: '',
  id: null
};

/* eslint-disable jsx-a11y/click-events-have-key-events */
const legalComparators$1 = [EQ, NE, GT, GE, LT, LE];

function dateParser(d) {
  return `${d.getUTCFullYear()}-${('0' + (d.getUTCMonth() + 1)).slice(-2)}-${('0' + d.getUTCDate()).slice(-2)}`;
}

class DateFilter extends Component {
  constructor(props) {
    super(props);
    this.timeout = null;
    this.comparators = props.comparators || legalComparators$1;
    this.applyFilter = this.applyFilter.bind(this);
    this.onChangeDate = this.onChangeDate.bind(this);
    this.onChangeComparator = this.onChangeComparator.bind(this);
  }

  componentDidMount() {
    const {
      getFilter
    } = this.props;
    const comparator = this.dateFilterComparator.value;
    const date = this.inputDate.value;

    if (comparator && date) {
      this.applyFilter(date, comparator, true);
    } // export onFilter function to allow users to access


    if (getFilter) {
      getFilter(filterVal => {
        const nullableFilterVal = filterVal || {
          date: null,
          comparator: null
        };
        this.dateFilterComparator.value = nullableFilterVal.comparator;
        this.inputDate.value = nullableFilterVal.date ? dateParser(nullableFilterVal.date) : null;
        this.applyFilter(nullableFilterVal.date, nullableFilterVal.comparator);
      });
    }
  }

  componentWillUnmount() {
    if (this.timeout) clearTimeout(this.timeout);
  }

  onChangeDate(e) {
    const comparator = this.dateFilterComparator.value;
    const filterValue = e.target.value;
    this.applyFilter(filterValue, comparator);
  }

  onChangeComparator(e) {
    const {
      value
    } = this.inputDate;
    const comparator = e.target.value;
    this.applyFilter(value, comparator);
  }

  getComparatorOptions() {
    const optionTags = [];
    const {
      withoutEmptyComparatorOption
    } = this.props;

    if (!withoutEmptyComparatorOption) {
      // eslint-disable-next-line jsx-a11y/control-has-associated-label
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: "-1"
      }));
    }

    for (let i = 0; i < this.comparators.length; i += 1) {
      optionTags.push( /*#__PURE__*/React.createElement("option", {
        key: i,
        value: this.comparators[i]
      }, this.comparators[i]));
    }

    return optionTags;
  }

  getDefaultComparator() {
    const {
      defaultValue,
      filterState
    } = this.props;

    if (filterState && filterState.filterVal) {
      return filterState.filterVal.comparator;
    }

    if (defaultValue && defaultValue.comparator) {
      return defaultValue.comparator;
    }

    return '';
  }

  getDefaultDate() {
    // Set the appropriate format for the input type=date, i.e. "YYYY-MM-DD"
    const {
      defaultValue,
      filterState
    } = this.props;

    if (filterState && filterState.filterVal && filterState.filterVal.date) {
      return dateParser(filterState.filterVal.date);
    }

    if (defaultValue && defaultValue.date) {
      return dateParser(new Date(defaultValue.date));
    }

    return '';
  }

  applyFilter(value, comparator, isInitial) {
    // if (!comparator || !value) {
    //  return;
    // }
    const {
      column,
      onFilter,
      delay
    } = this.props;

    const execute = () => {
      // Incoming value should always be a string, and the defaultDate
      // above is implemented as an empty string, so we can just check for that.
      // instead of parsing an invalid Date. The filter function will interpret
      // null as an empty date field
      const date = value === '' ? null : new Date(value);
      onFilter(column, FILTER_TYPE.DATE, isInitial)({
        date,
        comparator
      });
    };

    if (delay) {
      this.timeout = setTimeout(() => {
        execute();
      }, delay);
    } else {
      execute();
    }
  }

  render() {
    const {
      id,
      placeholder,
      column: {
        dataField,
        text
      },
      style,
      comparatorStyle,
      dateStyle,
      className,
      comparatorClassName,
      dateClassName
    } = this.props;
    const comparatorElmId = `date-filter-comparator-${dataField}${id ? `-${id}` : ''}`;
    const inputElmId = `date-filter-column-${dataField}${id ? `-${id}` : ''}`;
    return /*#__PURE__*/React.createElement("div", {
      onClick: e => e.stopPropagation(),
      className: `filter date-filter ${className}`,
      style: style
    }, /*#__PURE__*/React.createElement("label", {
      className: "filter-label",
      htmlFor: comparatorElmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Filter comparator"), /*#__PURE__*/React.createElement("select", {
      ref: n => this.dateFilterComparator = n,
      id: comparatorElmId,
      style: comparatorStyle,
      className: `date-filter-comparator form-control ${comparatorClassName}`,
      onChange: this.onChangeComparator,
      defaultValue: this.getDefaultComparator()
    }, this.getComparatorOptions())), /*#__PURE__*/React.createElement("label", {
      htmlFor: inputElmId
    }, /*#__PURE__*/React.createElement("span", {
      className: "sr-only"
    }, "Enter $", text), /*#__PURE__*/React.createElement("input", {
      ref: n => this.inputDate = n,
      id: inputElmId,
      className: `filter date-filter-input form-control ${dateClassName}`,
      style: dateStyle,
      type: "date",
      onChange: this.onChangeDate,
      placeholder: placeholder || `Enter ${text}...`,
      defaultValue: this.getDefaultDate()
    })));
  }

}

DateFilter.propTypes = {
  onFilter: PropTypes.func.isRequired,
  column: PropTypes.object.isRequired,
  id: PropTypes.string,
  filterState: PropTypes.object,
  delay: PropTypes.number,
  defaultValue: PropTypes.shape({
    date: PropTypes.oneOfType([PropTypes.object]),
    comparator: PropTypes.oneOf([...legalComparators$1, ''])
  }),

  /* eslint consistent-return: 0 */
  comparators: (props, propName) => {
    if (!props[propName]) {
      return;
    }

    for (let i = 0; i < props[propName].length; i += 1) {
      let comparatorIsValid = false;

      for (let j = 0; j < legalComparators$1.length; j += 1) {
        if (legalComparators$1[j] === props[propName][i] || props[propName][i] === '') {
          comparatorIsValid = true;
          break;
        }
      }

      if (!comparatorIsValid) {
        return new Error(`Date comparator provided is not supported.
          Use only ${legalComparators$1}`);
      }
    }
  },
  placeholder: PropTypes.string,
  withoutEmptyComparatorOption: PropTypes.bool,
  style: PropTypes.object,
  comparatorStyle: PropTypes.object,
  dateStyle: PropTypes.object,
  className: PropTypes.string,
  comparatorClassName: PropTypes.string,
  dateClassName: PropTypes.string,
  getFilter: PropTypes.func
};
DateFilter.defaultProps = {
  delay: 0,
  defaultValue: {
    date: undefined,
    comparator: ''
  },
  filterState: {},
  withoutEmptyComparatorOption: false,
  comparators: legalComparators$1,
  placeholder: undefined,
  style: undefined,
  className: '',
  comparatorStyle: undefined,
  comparatorClassName: '',
  dateStyle: undefined,
  dateClassName: '',
  id: null
};

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _classPrivateFieldBase(receiver, privateKey) {
  if (!Object.prototype.hasOwnProperty.call(receiver, privateKey)) {
    throw new TypeError("attempted to use private field on non-instance");
  }

  return receiver;
}

var id = 0;
function _classPrivateFieldKey(name) {
  return "__private_" + id++ + "_" + name;
}

const filterByText = _ => (data, dataField, {
  filterVal: userInput = '',
  comparator: _comparator = LIKE,
  caseSensitive
}, customFilterValue) => {
  // make sure filter value to be a string
  const filterVal = userInput.toString();
  return data.filter(row => {
    let cell = _.get(row, dataField);

    if (customFilterValue) {
      cell = customFilterValue(cell, row);
    }

    const cellStr = _.isDefined(cell) ? cell.toString() : '';

    if (_comparator === EQ) {
      return cellStr === filterVal;
    }

    if (caseSensitive) {
      return cellStr.includes(filterVal);
    }

    return cellStr.toLocaleUpperCase().indexOf(filterVal.toLocaleUpperCase()) !== -1;
  });
};
const filterByNumber = _ => (data, dataField, {
  filterVal: {
    comparator,
    number
  }
}, customFilterValue) => data.filter(row => {
  if (number === '' || !comparator) return true;

  let cell = _.get(row, dataField);

  if (customFilterValue) {
    cell = customFilterValue(cell, row);
  }

  switch (comparator) {
    case EQ:
      {
        return cell == number;
      }

    case GT:
      {
        return cell > number;
      }

    case GE:
      {
        return cell >= number;
      }

    case LT:
      {
        return cell < number;
      }

    case LE:
      {
        return cell <= number;
      }

    case NE:
      {
        return cell != number;
      }

    default:
      {
        console.error('Number comparator provided is not supported');
        return true;
      }
  }
});
const filterByDate = _ => (data, dataField, {
  filterVal: {
    comparator,
    date
  }
}, customFilterValue) => {
  if (!date || !comparator) return data;
  const filterDate = date.getUTCDate();
  const filterMonth = date.getUTCMonth();
  const filterYear = date.getUTCFullYear();
  return data.filter(row => {
    let valid = true;

    let cell = _.get(row, dataField);

    if (customFilterValue) {
      cell = customFilterValue(cell, row);
    }

    if (typeof cell !== 'object') {
      cell = new Date(cell);
    }

    const targetDate = cell.getUTCDate();
    const targetMonth = cell.getUTCMonth();
    const targetYear = cell.getUTCFullYear();

    switch (comparator) {
      case EQ:
        {
          if (filterDate !== targetDate || filterMonth !== targetMonth || filterYear !== targetYear) {
            valid = false;
          }

          break;
        }

      case GT:
        {
          if (cell <= date) {
            valid = false;
          }

          break;
        }

      case GE:
        {
          if (targetYear < filterYear) {
            valid = false;
          } else if (targetYear === filterYear && targetMonth < filterMonth) {
            valid = false;
          } else if (targetYear === filterYear && targetMonth === filterMonth && targetDate < filterDate) {
            valid = false;
          }

          break;
        }

      case LT:
        {
          if (cell >= date) {
            valid = false;
          }

          break;
        }

      case LE:
        {
          if (targetYear > filterYear) {
            valid = false;
          } else if (targetYear === filterYear && targetMonth > filterMonth) {
            valid = false;
          } else if (targetYear === filterYear && targetMonth === filterMonth && targetDate > filterDate) {
            valid = false;
          }

          break;
        }

      case NE:
        {
          if (filterDate === targetDate && filterMonth === targetMonth && filterYear === targetYear) {
            valid = false;
          }

          break;
        }

      default:
        {
          console.error('Date comparator provided is not supported');
          break;
        }
    }

    return valid;
  });
};
const filterByArray = _ => (data, dataField, {
  filterVal,
  comparator
}) => {
  if (filterVal.length === 0) return data;
  const refinedFilterVal = filterVal.filter(x => _.isDefined(x)).map(x => x.toString());
  return data.filter(row => {
    const cell = _.get(row, dataField);

    let cellStr = _.isDefined(cell) ? cell.toString() : '';

    if (comparator === EQ) {
      return refinedFilterVal.indexOf(cellStr) !== -1;
    }

    cellStr = cellStr.toLocaleUpperCase();
    return refinedFilterVal.some(item => cellStr.indexOf(item.toLocaleUpperCase()) !== -1);
  });
};
const filterFactory = _ => filterType => {
  switch (filterType) {
    case FILTER_TYPE.MULTISELECT:
      return filterByArray(_);

    case FILTER_TYPE.NUMBER:
      return filterByNumber(_);

    case FILTER_TYPE.DATE:
      return filterByDate(_);

    case FILTER_TYPE.TEXT:
    case FILTER_TYPE.SELECT:
    default:
      // Use `text` filter as default filter
      return filterByText(_);
  }
};
const filters = (data, columns, _) => (currFilters, clearFilters = {}) => {
  const factory = filterFactory(_);

  const filterState = _objectSpread2(_objectSpread2({}, clearFilters), currFilters);

  let result = data;
  let filterFn;
  Object.keys(filterState).forEach(dataField => {
    let currentResult;
    let filterValue;
    let customFilter;

    for (let i = 0; i < columns.length; i += 1) {
      if (columns[i].dataField === dataField) {
        filterValue = columns[i].filterValue;

        if (columns[i].filter) {
          customFilter = columns[i].filter.props.onFilter;
        }

        break;
      }
    }

    if (clearFilters[dataField] && customFilter) {
      currentResult = customFilter(clearFilters[dataField].filterVal, result);

      if (typeof currentResult !== 'undefined') {
        result = currentResult;
      }
    } else {
      const filterObj = filterState[dataField];
      filterFn = factory(filterObj.filterType);

      if (customFilter) {
        currentResult = customFilter(filterObj.filterVal, result);
      }

      if (typeof currentResult === 'undefined') {
        result = filterFn(result, dataField, filterObj, filterValue);
      } else {
        result = currentResult;
      }
    }
  });
  return result;
};

var createContext = ((_, isRemoteFiltering, handleFilterChange) => {
  const FilterContext = /*#__PURE__*/React.createContext();

  var _data = _classPrivateFieldKey("data");

  class FilterProvider extends React.Component {
    constructor(_props) {
      super(_props);
      Object.defineProperty(this, _data, {
        writable: true,
        value: this.props.data
      });

      this.onFilter = (column, filterType, initialize = false) => filterVal => {
        // watch out here if migration to context API, #334
        const currFilters = _objectSpread2({}, this.currFilters);

        this.clearFilters = {};
        const {
          dataField,
          filter
        } = column;
        const needClearFilters = !_.isDefined(filterVal) || filterVal === '' || filterVal.length === 0;

        if (needClearFilters) {
          delete currFilters[dataField];
          this.clearFilters = {
            [dataField]: {
              clear: true,
              filterVal
            }
          };
        } else {
          // select default comparator is EQ, others are LIKE
          const {
            comparator = filterType === FILTER_TYPE.SELECT ? EQ : LIKE,
            caseSensitive = false
          } = filter.props;
          currFilters[dataField] = {
            filterVal,
            filterType,
            comparator,
            caseSensitive
          };
        }

        this.currFilters = currFilters;

        if (isRemoteFiltering()) {
          if (!initialize) {
            handleFilterChange(this.currFilters);
          }

          return;
        }

        this.doFilter();
      };

      this.onExternalFilter = (column, filterType) => value => {
        this.onFilter(column, filterType)(value);
      };

      this.doFilter = (props = this.props, ignoreEmitDataChange = false) => {
        const {
          dataChangeListener,
          data,
          columns,
          filter
        } = props;
        const result = filters(data, columns, _)(this.currFilters, this.clearFilters);

        if (filter.afterFilter) {
          filter.afterFilter(result, this.currFilters);
        }

        this.data = result;

        if (dataChangeListener && !ignoreEmitDataChange) {
          this.isEmitDataChange = true;
          dataChangeListener.emit('filterChanged', result.length);
        } else {
          this.isEmitDataChange = false;
          this.forceUpdate();
        }
      };

      this.currFilters = {};
      this.clearFilters = {};
      this.isEmitDataChange = false;
    } // eslint-disable-next-line react/sort-comp


    get data() {
      return this.props.data;
    }

    set data(value) {
      _classPrivateFieldBase(this, _data)[_data] = value;
    }

    componentDidMount() {
      if (isRemoteFiltering() && Object.keys(this.currFilters).length > 0) {
        handleFilterChange(this.currFilters);
      }
    }

    componentDidUpdate(prevProps) {
      if (!_.isEqual(this.props.data, prevProps.data) && !isRemoteFiltering()) {
        this.doFilter(this.props, this.isEmitDataChange);
      }
    }

    getFiltered() {
      // eslint-disable-next-line react/destructuring-assignment
      return this.data;
    }

    render() {
      const {
        children
      } = this.props;
      return /*#__PURE__*/React.createElement(FilterContext.Provider, {
        value: {
          data: this.data,
          onFilter: this.onFilter,
          onExternalFilter: this.onExternalFilter,
          currFilters: this.currFilters
        }
      }, children);
    }

  }

  FilterProvider.propTypes = {
    data: PropTypes$1.array.isRequired,
    columns: PropTypes$1.array.isRequired,
    dataChangeListener: PropTypes$1.object
  };
  return {
    Provider: FilterProvider,
    Consumer: FilterContext.Consumer
  };
});

var index = ((options = {}) => ({
  createContext,
  options
}));
const FILTER_TYPES = FILTER_TYPE;
const Comparator = Comparison;
const textFilter = (props = {}, Filter = TextFilter) => ({
  Filter,
  props
});
const selectFilter = (props = {}, Filter = SelectFilter) => ({
  Filter,
  props
});
const multiSelectFilter = (props = {}, Filter = MultiSelectFilter) => ({
  Filter,
  props
});
const numberFilter = (props = {}, Filter = NumberFilter) => ({
  Filter,
  props
});
const dateFilter = (props = {}, Filter = DateFilter) => ({
  Filter,
  props
});
const customFilter = (props = {}) => ({
  props
});

export default index;
export { Comparator, FILTER_TYPES, customFilter, dateFilter, multiSelectFilter, numberFilter, selectFilter, textFilter };
